<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Definition;

use Dedi\SyliusMigrationPlugin\Dto\AfterMigrateDefinition;
use Dedi\SyliusMigrationPlugin\Dto\BeforeMigrateDefinition;
use Dedi\SyliusMigrationPlugin\Dto\DateFieldsMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\FilterMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\IdentifierMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\LimitMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\OrderMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\SearchMappingDefinition;

class TableMappingDefinition
{
    private readonly string $targetTableName;

    private readonly string $sourceTableName;

    private readonly array $identifierMappingDefinitions;

    private readonly array $columnMappingDefinitions;

    private readonly string $sql;

    private readonly string $sourcePrimaryKey;

    private readonly array $searchMapping;

    private readonly array $filterMapping;

    private readonly array $logginColumns;

    private readonly LimitMappingDefinition $limitMappingDefinition;

    private readonly ?OrderMappingDefinition $orderMappingDefinition;

    private readonly ?BeforeMigrateDefinition $beforeMigrateDefinition;

    private readonly ?AfterMigrateDefinition $afterMigrateDefinition;

    private readonly ?array $errorHandlers;

    private readonly ?string $handler;

    private readonly DateFieldsMappingDefinition $dateFieldsMappingDefinition;

    private readonly ?string $group;

    private readonly ?array $truncateMappingDefinition;

    public function __construct(
        string $targetTableName,
        string $sourceTableName,
        string $sql,
        string $sourcePrimaryKey,
        array $identifierMappingDefinitions,
        array $columnMappingDefinitions,
        array $searchMapping,
        array $filterMapping,
        array $logginColumns,
        LimitMappingDefinition $limitMappingDefinition,
        DateFieldsMappingDefinition $dateFieldsMappingDefinition,
        ?OrderMappingDefinition $orderMappingDefinition = null,
        ?array $truncateMappingDefinition = null,
        ?BeforeMigrateDefinition $beforeMigrateDefinition = null,
        ?AfterMigrateDefinition $afterMigrateDefinition = null,
        ?string $handler = null,
        ?array $errorHandlers = [],
        ?string $group = null,
    ) {
        $this->targetTableName = $targetTableName;
        $this->sourceTableName = $sourceTableName;
        $this->sourcePrimaryKey = $sourcePrimaryKey;
        $this->columnMappingDefinitions = $columnMappingDefinitions;
        $this->sql = $sql;
        $this->searchMapping = $searchMapping;
        $this->filterMapping = $filterMapping;
        $this->orderMappingDefinition = $orderMappingDefinition;
        $this->logginColumns = $logginColumns;
        $this->identifierMappingDefinitions = $identifierMappingDefinitions;
        $this->beforeMigrateDefinition = $beforeMigrateDefinition;
        $this->afterMigrateDefinition = $afterMigrateDefinition;
        $this->limitMappingDefinition = $limitMappingDefinition;
        $this->handler = $handler;
        $this->errorHandlers = $errorHandlers;
        $this->dateFieldsMappingDefinition = $dateFieldsMappingDefinition;
        $this->group = $group;
        $this->truncateMappingDefinition = $truncateMappingDefinition;
    }

    public function getLimitMappingDefinition(): LimitMappingDefinition
    {
        return $this->limitMappingDefinition;
    }

    public function getTargetTableName(): string
    {
        return $this->targetTableName;
    }

    public function getSourceTableName(): string
    {
        return $this->sourceTableName;
    }

    /**
     * @return array|ColumnMappingDefinition[]
     */
    public function getColumnMappingDefinitions(): array
    {
        return $this->columnMappingDefinitions;
    }

    public function getSql(): string
    {
        return $this->sql;
    }

    public function getSourcePrimaryKey(): string
    {
        return $this->sourcePrimaryKey;
    }

    /**
     * @return array|IdentifierMappingDefinition[]
     */
    public function getIdentifierMappingDefinitions(): array
    {
        return $this->identifierMappingDefinitions;
    }

    public function getBeforeMigrateDefinition(): ?BeforeMigrateDefinition
    {
        return $this->beforeMigrateDefinition;
    }

    public function getAfterMigrateDefinition(): ?AfterMigrateDefinition
    {
        return $this->afterMigrateDefinition;
    }

    public function hasColumnMapping(string $columnName): bool
    {
        /** @var ColumnMappingDefinition $columnMappingDefinition */
        foreach ($this->columnMappingDefinitions as $columnMappingDefinition) {
            if ($columnName === $columnMappingDefinition->getTargetColumName()) {
                return true;
            }
        }

        return false;
    }

    /** @return array|SearchMappingDefinition[] */
    public function getSearchMapping(): array
    {
        return $this->searchMapping;
    }

    public function hasSearchMapping(): bool
    {
        return 0 < count($this->searchMapping);
    }

    /** @return array|FilterMappingDefinition[] */
    public function getFilterMapping(): array
    {
        return $this->filterMapping;
    }

    public function hasFilterMapping(): bool
    {
        return 0 < count($this->filterMapping);
    }

    public function getOrderMappingDefinition(): ?OrderMappingDefinition
    {
        return $this->orderMappingDefinition;
    }

    public function getLoggingColumns(): array
    {
        return $this->logginColumns;
    }

    public function getHandler(): ?string
    {
        return $this->handler;
    }

    public function getErrorHandlers(): array
    {
        return $this->errorHandlers;
    }

    public function getDateFieldsMappingDefinition(): DateFieldsMappingDefinition
    {
        return $this->dateFieldsMappingDefinition;
    }

    public function getGroup(): ?string
    {
        return $this->group;
    }

    public function getTruncateMappingDefinition(): ?array
    {
        return $this->truncateMappingDefinition;
    }
}
